<?php
/*
	Plugin:		Style Usernames Plugin
	Version:	1.2
	Author:		Lukasz "LukasAMD" Tkacz 
	Date:		  17.02.2011
*/

if (!defined("IN_MYBB"))
{
	die("Direct initialization of this file is not allowed.<br /><br />Please make sure IN_MYBB is defined.");
}


// Add hooks
$plugins->add_hook('global_start', 'styleUsernames::initCache');
$plugins->add_hook('build_forumbits_forum', 'styleUsernames::buildForumbits');
$plugins->add_hook('forumdisplay_announcement', 'styleUsernames::forumdisplayAnnouncement');
$plugins->add_hook('forumdisplay_thread', 'styleUsernames::forumdisplayThread');

$plugins->add_hook('search_results_thread', 'styleUsernames::searchThread');
$plugins->add_hook('search_results_post', 'styleUsernames::searchPost'); 
          


// Stanard MyBB function with informations about plugin
function styleusernames_info()
{
	global $lang;

	$lang->load("styleusernames");

	return Array(
		'name' => $lang->styleUsernamesName,
		'description' => $lang->styleUsernamesDesc,
		'website' => 'http://lukasztkacz.com',
		'author' => 'Lukasz "LukasAMD" Tkacz',
		'authorsite' => 'http://lukasztkacz.com',
		'version' => '1.2',
		'guid' => '6f0320c2e4b0f78792dd626a60ea33d1',
		'compatibility' => '16*'
	);
}

// Standard MyBB installation functions
function styleusernames_install() 
{
  global $mybb, $db, $lang;
  
	// Unistall plugin
	$db->delete_query('settinggroups', "name = 'styleUsernames'");
	$db->delete_query('settings', "name = 'styleUsernamesCacheMethod'");
	
	if ($db->index_exists('users', 'postnum'))
	{
    $sql = 'ALTER TABLE ' . TABLE_PREFIX . 'users DROP INDEX postnum'; 
    $db->query($sql);
  }


  $result = $db->simple_select('settinggroups', 'MAX(disporder) AS max_disporder');
  $max_disporder = $db->fetch_field($result, 'max_disporder');

	$settings_group = array(
		'gid'			=> 'NULL',
		'name'			=> 'styleUsernames',
		'title'			=> $db->escape_string($lang->styleUsernamesName),
		'description'	=> $db->escape_string($lang->styleUsernamesSettingGroupDesc),
		'disporder'		=> $max_disporder + 1,
		'isdefault'		=> '0'
	);
	$db->insert_query('settinggroups', $settings_group);
	$gid = (int) $db->insert_id();

	$setting = array(
		'sid'			=> 'NULL',
		'name'			=> 'styleUsernamesCacheMethod',
		'title'			=> $db->escape_string($lang->styleUsernamesCacheMethod),
		'description'	=> $db->escape_string($lang->styleUsernamesCacheMethodDesc),
		'optionscode'	=> 'onoff',
		'value'			=> '0',
		'disporder'		=> '1',
		'gid'			=> $gid
	);
	$db->insert_query('settings', $setting);

	if (!$db->index_exists('users', 'postnum'))
	{
    $sql = 'ALTER TABLE ' . TABLE_PREFIX . 'users ADD INDEX (postnum)'; 
    $db->query($sql);
  }
	
	rebuild_settings();
}


function styleusernames_is_installed() 
{
  global $mybb;
  
  return (isset($mybb->settings['styleUsernamesCacheMethod']));
}


function styleusernames_uninstall() 
{
  global $mybb, $db;
  
	$db->delete_query('settinggroups', "name = 'styleUsernames'");
	$db->delete_query('settings', "name = 'styleUsernamesCacheMethod'");
  
	if ($db->index_exists('users', 'postnum'))
	{
    $sql = 'ALTER TABLE ' . TABLE_PREFIX . 'users DROP INDEX postnum'; 
    $db->query($sql);
  }

  rebuild_settings();
}
// END - Standard MyBB installation functions




// Style Usernames Plugin Class
class styleUsernames
{
  
  // Main Style Usernames function. If full cache is enabled, make it now
  public static function initCache()
  {
    global $mybb, $su_cache;
  
    $allowedScripts = array('index.php', 'search.php', 'forumdisplay.php');
  
    if ($mybb->settings['styleUsernamesCacheMethod'] && in_array(THIS_SCRIPT, $allowedScripts))
    {
      global $db;
    
      $result = $db->simple_select('users', 'uid, username, usergroup, displaygroup', 'postnum > 0');
      while ($row = $db->fetch_array($result))
      {
        $su_cache[$row['uid']] = format_name($row['username'], $row['usergroup'], $row['displaygroup']);
      }
    }
    return true;
  } 
  
  
  // Style usernames on forums list
  public static function buildForumbits(&$forum)
  {
  	global $db, $mybb, $su_cache;

  	if (isset($su_cache[$forum['lastposteruid']]))
  	{
      $forum['lastposter'] = $su_cache[$forum['lastposteruid']]; 
    }
    else
    {
      $result = $db->simple_select('users', 'usergroup, displaygroup', "uid = '" . $forum['lastposteruid'] . "'");
      $row = $db->fetch_array($result);
  	
      $forum['lastposter'] = format_name($forum['lastposter'], $row['usergroup'], $row['displaygroup']);
      $su_cache[$forum['lastposteruid']] = $forum['lastposter'];  
    }
  }
  
  
  // Style usernames on announcements list
  public static function forumdisplayAnnouncement()
  {
  	global $db, $announcement, $su_cache;
  
  	if (isset($su_cache[$announcement['uid']]))
  	{
      $announcement['username'] = $su_cache[$announcement['uid']]; 
    }
    else
    {
      $result = $db->simple_select('users', 'usergroup, displaygroup', "uid = '" . $announcement['uid'] . "'");
      $row = $db->fetch_array($result);
  	
      $announcement['username'] = format_name($announcement['username'], $row['usergroup'], $row['displaygroup']);
      $su_cache[$announcement['uid']] = $announcement['username'];  
    }
  
  	// Format announcement author name
  	$announcement['profilelink'] = build_profile_link($announcement['username'], $announcement['uid']);	
  }
  
  
  // Style usernames on topics list
  public static function forumdisplayThread()
  {
  	global $db, $thread, $su_cache;
  
  	// Format thread author name
  	if ($thread['username'])
  	{
    	if (isset($su_cache[$thread['uid']]))
    	{
        $thread['username'] = $su_cache[$thread['uid']]; 
      }
      else
      {
        $result = $db->simple_select('users', 'usergroup, displaygroup', "uid = '" . $thread['uid'] . "'");
        $row = $db->fetch_array($result);
    	
        $thread['username'] = format_name($thread['username'], $row['usergroup'], $row['displaygroup']);
        $su_cache[$thread['uid']] = $thread['username'];  
      }
  	}
  	
  	// Format lastposter name
  	if ($thread['lastposteruid'] != 0)
  	{
    	if (isset($su_cache[$thread['lastposteruid']]))
    	{
        $thread['lastposter'] = $su_cache[$thread['lastposteruid']]; 
      }
      else
      {
        $result = $db->simple_select('users', 'usergroup, displaygroup', "uid = '" . $thread['lastposteruid'] . "'");
        $row = $db->fetch_array($result);
    	
        $thread['lastposter'] = format_name($thread['lastposter'], $row['usergroup'], $row['displaygroup']);
        $su_cache[$thread['lastposteruid']] = $thread['lastposter'];  
      }
  	}
  }

  
  // Style usernames on topics list (search results)
  public static function searchThread()
  {
  	global $db, $thread, $lastposterlink, $su_cache;
  	
  	// Format thread author name
  	if ($thread['username'])
  	{
    	if (isset($su_cache[$thread['uid']]))
    	{
        $thread['username'] = $su_cache[$thread['uid']]; 
      }
      else
      {
        $result = $db->simple_select('users', 'usergroup, displaygroup', "uid = '" . $thread['uid'] . "'");
        $row = $db->fetch_array($result);
    	
        $thread['username'] = format_name($thread['username'], $row['usergroup'], $row['displaygroup']);
        $su_cache[$thread['uid']] = $thread['username'];  
      }
      
  	  $thread['profilelink'] = build_profile_link($thread['username'], $thread['uid']);
  	}
  	
  	// Format lastposter name
  	if ($thread['lastposteruid'] != 0)
  	{
    	if (isset($su_cache[$thread['lastposteruid']]))
    	{
        $thread['lastposter'] = $su_cache[$thread['lastposteruid']]; 
      }
      else
      {
        $result = $db->simple_select('users', 'usergroup, displaygroup', "uid = '" . $thread['lastposteruid'] . "'");
        $row = $db->fetch_array($result);
    	
        $thread['lastposter'] = format_name($thread['lastposter'], $row['usergroup'], $row['displaygroup']);
        $su_cache[$thread['lastposteruid']] = $thread['lastposter'];  
      }
      
      $lastposterlink = build_profile_link($thread['lastposter'], $thread['lastposteruid']);
  	}
  }


  // Style usernames on posts list (search results)
  public static function searchPost()
  {
  	global $db, $post, $su_cache;
  	
  	// Format post author name
  	if ($post['uid'])
  	{
    	if (isset($su_cache[$post['uid']]))
    	{
        $post['username'] = $su_cache[$post['uid']]; 
      }
      else
      {
        $result = $db->simple_select("users", "usergroup, displaygroup", "uid = '" . $post['uid'] . "'");
        $row = $db->fetch_array($result);
    	
        $post['username'] = format_name($post['username'], $row['usergroup'], $row['displaygroup']);
        $su_cache[$post['uid']] = $post['username'];  
      }
      
  	  $post['profilelink'] = build_profile_link($post['username'], $post['uid']);
  	}
  } 
  
  
  // Additional method for manualy get username style
  public static function getUser($uid)
  {
  	global $db, $mybb, $su_cache;

  	if (isset($su_cache[$uid]))
  	{
      return $su_cache[$uid]; 
    }
    else
    {
      $result = $db->simple_select('users', 'username, usergroup, displaygroup', "uid = '" . $uid . "'");
      $row = $db->fetch_array($result);
  	
      $userName = format_name($row['username'], $row['usergroup'], $row['displaygroup']);
      $su_cache[$forum['lastposteruid']] = $userName;
      return $userName;  
    }
  }
}
?>