<?php
/**
 * Build a comma separated list of the forums this user cannot search
 *
 * @param int The parent ID to build from
 * @param int First rotation or not (leave at default)
 * @return return a CSV list of forums the user cannot search
 */
function unreadposts_get_unsearchable_forums($pid="0", $first=1)
{
	global $db, $forum_cache_unread, $permissioncache, $mybb, $unsearchableforums, $unsearchable, $templates;
	
	$pid = intval($pid);
	
	if(!is_array($forum_cache_unread))
	{
		// Get Forums
		$query = $db->simple_select("forums", "fid,parentlist,password,active", '', array('order_by' => 'pid, disporder'));
		while($forum = $db->fetch_array($query))
		{
			$forum_cache_unread[$forum['fid']] = $forum;
		}
	}
	if(!is_array($permissioncache))
	{
		$permissioncache = forum_permissions();
	}
	foreach($forum_cache_unread as $fid => $forum)
	{
		if($permissioncache[$forum['fid']])
		{
			$perms = $permissioncache[$forum['fid']];
		}
		else
		{
			$perms = $mybb->usergroup;
		}

		$pwverified = 1;
		if($forum['password'] != '')
		{
			if($mybb->cookies['forumpass'][$forum['fid']] != md5($mybb->user['uid'].$forum['password']))
			{
				$pwverified = 0;
			}
		}

		$parents = explode(",", $forum['parentlist']);
		if(is_array($parents))
		{
			foreach($parents as $parent)
			{
				if($forum_cache_unread[$parent]['active'] == 0)
				{
					$forum['active'] = 0;
				}
			}
		}

		if($perms['canview'] != 1 || $perms['cansearch'] != 1 || $pwverified == 0 || $forum['active'] == 0)
		{
			if($unsearchableforums)
			{
				$unsearchableforums .= ",";
			}
			$unsearchableforums .= "'{$forum['fid']}'";
		}
	}
	$unsearchable = $unsearchableforums;
	
	// Get our unsearchable password protected forums
	$pass_protected_forums = unreadposts_get_password_protected_forums();
	
	if($unsearchable && $pass_protected_forums)
	{
		$unsearchable .= ",";
	}
	
	if($pass_protected_forums)
	{
		$unsearchable .= implode(",", $pass_protected_forums);
	}
	
	return $unsearchable;
}

/**
 * Build a array list of the forums this user cannot search due to password protection
 *
 * @param int the fids to check (leave null to check all forums)
 * @return return a array list of password protected forums the user cannot search
 */
function unreadposts_get_password_protected_forums($fids=array())
{
	global $forum_cache_unread, $mybb;
	
	if(!is_array($fids))
	{
		return false;
	}
	
	if(!is_array($forum_cache_unread))
	{
		$forum_cache_unread = cache_forums();
		if(!$forum_cache_unread)
		{
			return false;
		}
	}
	
	if(empty($fids))
	{
		$fids = array_keys($forum_cache_unread);
	}

	$pass_fids = array();
	foreach($fids as $fid)
	{
		if(empty($forum_cache_unread[$fid]['password']))
		{
			continue;
		}
		
		if(md5($mybb->user['uid'].$forum_cache_unread[$fid]['password']) != $mybb->cookies['forumpass'][$fid])
		{
			$pass_fids[] = $fid;
			$child_list = get_child_list($fid);
		}
		
		if(is_array($child_list))
		{
			$pass_fids = array_merge($pass_fids, $child_list);
		}
	}
	return array_unique($pass_fids);
}
?>