<?php
/*
	Plugin:		UnreadPosts Plugin
	Version:	1.8
	Author:		Lukasz "LukasAMD" Tkacz 
	Date:		18.11.2010
	
	Free for non-commercial purposes!
	License: GNU Public License
	http://opensource.org/licenses/gpl-3.0.html
*/


if (!defined("IN_MYBB"))
{
	die("Direct initialization of this file is not allowed.<br /><br />Please make sure IN_MYBB is defined.");
}


// Add hooks
$plugins->add_hook('search_start', 'unreadposts_search');
$plugins->add_hook('global_start', 'unreadposts_link');
$plugins->add_hook('member_do_register_end', 'unreadposts_updatemark');
$plugins->add_hook('misc_markread_end', 'unreadposts_updatemark');


// Stanard MyBB function with informations about plugin
function unreadsposts_info()
{
	global $lang;

	$lang->load("unreadsposts");

	return Array(
		'name' => $lang->unreadsposts_plugin_Name,
		'description' => $lang->unreadsposts_plugin_Desc,
		'website' => 'http://lukasamd.pl',
		'author' => $lang->unreadsposts_plugin_Author,
		'authorsite' => 'http://lukasamd.pl',
		'version' => '1.8',
		'guid' => '2817698896addbff5ef705626b7e1a36',
		'compatibility' => '14*,16*'
	);
}

// Standard MyBB installation functions
function unreadsposts_install() 
{
  global $mybb, $db, $lang;
  
	// Unistall plugin
	$db->delete_query('settinggroups', "name = 'unreadsposts_plugin'");
	$db->delete_query('settings', "name = 'unreadsposts_plugin_exceptions'");
  $db->delete_query('settings', "name = 'unreadsposts_plugin_counter'");
  $db->delete_query('settings', "name = 'unreadsposts_plugin_counter_hide'");
  
	
  $result = $db->simple_select('settinggroups', 'MAX(disporder) AS max_disporder');
  $max_disporder = $db->fetch_field($result, 'max_disporder');

	$settings_group = array(
		'gid'			=> 'NULL',
		'name'			=> 'unreadsposts_plugin',
		'title'			=> $db->escape_string($lang->unreadsposts_plugin_settinggroup_name),
		'description'	=> $db->escape_string($lang->unreadsposts_plugin_settinggroup_description),
		'disporder'		=> $max_disporder + 1,
		'isdefault'		=> '0'
	);
	$db->insert_query('settinggroups', $settings_group);
	$gid = (int) $db->insert_id();

	$setting = array(
		'sid'			=> 'NULL',
		'name'			=> 'unreadsposts_plugin_exceptions',
		'title'			=> $db->escape_string($lang->unreadsposts_plugin_setting_exceptions),
		'description'	=> $db->escape_string($lang->unreadsposts_plugin_setting_exceptions_desc),
		'optionscode'	=> 'text',
		'value'			=> '',
		'disporder'		=> '1',
		'gid'			=> $gid
	);
	$db->insert_query('settings', $setting);
	
	$setting = array(
		'sid'			=> 'NULL',
		'name'			=> 'unreadsposts_plugin_counter',
		'title'			=> $db->escape_string( $lang->unreadsposts_plugin_setting_counter),
		'description'	=> $db->escape_string($lang->unreadsposts_plugin_setting_counter_desc),
		'optionscode'	=> 'onoff',
		'value'			=> '0',
		'disporder'		=> '2',
		'gid'			=> $gid
	);
	$db->insert_query('settings', $setting);

	$setting = array(
		'sid'			=> 'NULL',
		'name'			=> 'unreadsposts_plugin_counter_hide',
		'title'			=> $db->escape_string( $lang->unreadsposts_plugin_setting_counter_hide),
		'description'	=> $db->escape_string($lang->unreadsposts_plugin_setting_counter_hide_desc),
		'optionscode'	=> 'onoff',
		'value'			=> '0',
		'disporder'		=> '3',
		'gid'			=> $gid
	);
	$db->insert_query('settings', $setting);	
	rebuild_settings();
	
  
  // Add lastmark column into users table
  $sql = "ALTER TABLE " . TABLE_PREFIX . "users ADD COLUMN lastmark INT(10) NOT NULL DEFAULT '0'";
  $db->query($sql);
  
  $sql = "UPDATE " . TABLE_PREFIX . "users
          SET lastmark = regdate";
  $db->query($sql);
}

function unreadsposts_is_installed() 
{
  global $mybb;
  
  return (isset($mybb->user['lastmark']));
}

function unreadsposts_uninstall() 
{
  global $mybb, $db;
  
  $sql = "ALTER TABLE " . TABLE_PREFIX . "users DROP lastmark";
  $db->query($sql);
  
	$db->delete_query('settinggroups', "name = 'unreadsposts_plugin'");
	$db->delete_query('settings', "name = 'unreadsposts_plugin_exceptions'");
  $db->delete_query('settings', "name = 'unreadsposts_plugin_counter'");
  $db->delete_query('settings', "name = 'unreadsposts_plugin_counter_hide'");
  
  rebuild_settings();
}
// END - Standard MyBB installation functions


// START - Standard MyBB activation functions
function unreadsposts_activate() 
{
  global $lang;
	include MYBB_ROOT . '/inc/adminfunctions_templates.php';

  find_replace_templatesets('header_welcomeblock_member', '#' . preg_quote('{$lang->welcome_todaysposts}</a>') . '#', '{$lang->welcome_todaysposts}</a>{$unreadspost}');
}


function unreadsposts_is_activated() 
{
  global $mybb;
  
  return (isset($mybb->user['lastmark']));
}

function unreadsposts_deactivate() 
{
  global $lang, $db;
	include MYBB_ROOT . '/inc/adminfunctions_templates.php';

  find_replace_templatesets('header_welcomeblock_member', '#' . preg_quote('{$unreadspost}') . '#', '');
}
// END - Standard MyBB activation functions



// MyBB Template functions doesn't work in previous version of this plugin (or my regex is so stupid)
// In effect, I decided to use global hook to add new link for browsing unread posts
function unreadposts_link()
{
  global $lang, $unreadspost, $mybb;
  $lang->load("unreadsposts"); 

  if ($mybb->settings['unreadsposts_plugin_counter'] && $mybb->user['uid'])
  {
    global $db;
    
    require_once MYBB_ROOT . '/inc/plugins/unreadsposts-inc.php';
    $where_sql = $unreadspost = '';
  
    // Get a exceptions forums list
    if ($mybb->settings['unreadsposts_plugin_exceptions'] != '')
    {
      $exceptions_list = explode(',', $mybb->settings['unreadsposts_plugin_exceptions']);
      $exceptions_count = sizeof($exceptions_list);
      
      $exceptions_list_filtered = array_filter($exceptions_list, 'clean_exceptions');
    
      if (sizeof($exceptions_list_filtered) > 0)
      {
        $where_sql .= " AND t.fid NOT IN (" . implode(',', $exceptions_list_filtered) . ")";
      }
    }
    
    // Get a unsearchtable forums list
    $unsearchforums = unreadposts_get_unsearchable_forums();
    if($unsearchforums)
    {
      $where_sql .= " AND t.fid NOT IN ($unsearchforums)";
    }
    
    // Group permission compatibility for >= MyBB 1.6
    if ($version['version_code'] >= 1600)
    {
    	$onlyusfids = array();
    
    	// Check group permissions if we can't view threads not started by us
    	$group_permissions = forum_permissions();
    	foreach($group_permissions as $fid => $forum_permissions)
    	{
    		if($forum_permissions['canonlyviewownthreads'] == 1)
    		{
    			$onlyusfids[] = $fid;
    		}
    	}
    	if(!empty($onlyusfids))
    	{
    		$where_sql .= " AND ((t.fid IN(".implode(',', $onlyusfids).") AND t.uid='{$mybb->user['uid']}') OR t.fid NOT IN(".implode(',', $onlyusfids)."))"; 
    	}
    }

    // Make a query to search topics with unread posts
    $sql = "SELECT COUNT(p.pid) AS unread_count
            FROM " . TABLE_PREFIX . "posts p
            INNER JOIN " . TABLE_PREFIX . "threads t ON (p.tid = t.tid)
            LEFT JOIN " . TABLE_PREFIX . "threadsread tr ON (tr.uid = " . $mybb->user['uid'] . " AND t.tid = tr.tid) 
            LEFT JOIN " . TABLE_PREFIX . "forumsread fr ON (fr.uid = " . $mybb->user['uid'] . " AND t.fid = fr.fid) 
            WHERE t.visible = 1 
              AND t.closed NOT LIKE 'moved|%' 
              AND p.dateline > IFNULL(tr.dateline," . $mybb->user['lastmark'] . ") 
              AND p.dateline > IFNULL(fr.dateline," . $mybb->user['lastmark'] . ") 
              AND p.dateline > " . $mybb->user['lastmark'] . $where_sql . " 
            ORDER BY t.dateline DESC";
    $query = $db->query($sql);
    $unread_count = $db->fetch_field($query, 'unread_count');
    
    // Check numer of unread and couter visible setting
    if ($unread_count > 0 || $mybb->settings['unreadsposts_plugin_counter_hide'] == 0)
    {
      $unreadspost =  ' | <a href="' . $mybb->settings['bburl'] . '/search.php?action=unreads">' . $lang->unreadsposts_plugin_Link . ' (' . $unread_count . ')</a>'; 
    }
  }
  else
  {
    $unreadspost =  ' | <a href="' . $mybb->settings['bburl'] . '/search.php?action=unreads">' . $lang->unreadsposts_plugin_Link . '</a>';
  }

  return true;
} 


// This function is called when users register and when click "Mark All Forums Read"
// We need lastmark timestamp to search additional posts 
function unreadposts_updatemark()
{
  global $db, $user_info, $mybb;

  $uid = (isset($user_info['uid'])) ? $user_info['uid'] : $mybb->user['uid'];

  $sql = "UPDATE " . TABLE_PREFIX . "users
          SET lastmark = '" . time() . "'
          WHERE uid = '" . $uid . "'";
  $db->query($sql);
  return true;
} 


// This function is called, when users use view unread posts function
// It search posts using threadsread forumsread (tables) and lastmark (field) 
function unreadposts_search()
{
  global $mybb, $db, $lang, $templates, $theme, $plugins, $version;

  if($mybb->input['action'] == 'unreads' && $mybb->user['uid'])
  {
    // Make a query to search topics with unread posts
    $sql = "SELECT t.tid
            FROM " . TABLE_PREFIX . "threads t
            LEFT JOIN " . TABLE_PREFIX . "threadsread tr ON (tr.uid = " . $mybb->user['uid'] . " AND t.tid = tr.tid) 
            LEFT JOIN " . TABLE_PREFIX . "forumsread fr ON (fr.uid = " . $mybb->user['uid'] . " AND t.fid = fr.fid) 
            WHERE t.visible = 1 
            AND t.closed NOT LIKE 'moved|%' 
            AND t.lastpost > IFNULL(tr.dateline," . $mybb->user['lastmark'] . ") AND t.lastpost > IFNULL(fr.dateline," . $mybb->user['lastmark'] . ") AND t.lastpost > " . $mybb->user['lastmark'] . "
            ORDER BY t.dateline DESC";
    $query = $db->query($sql);
    
    // Build a unread topics list 
    while($row = $db->fetch_array($query))
    {
      $tids[] = $row['tid']; 
    }
    
    // Get a exceptions forums list
    if ($mybb->settings['unreadsposts_plugin_exceptions'] != '')
    {
      $exceptions_list = explode(',', $mybb->settings['unreadsposts_plugin_exceptions']);
      $exceptions_count = sizeof($exceptions_list);
      
      $exceptions_list_filtered = array_filter($exceptions_list, 'clean_exceptions');
    
      if (sizeof($exceptions_list_filtered) > 0)
      {
        $where_sql .= "t.fid NOT IN (" . implode(',', $exceptions_list_filtered) . ") AND ";
      }
    }
    
    // Get a unsearchtable forums list
    $unsearchforums = get_unsearchable_forums();
    if($unsearchforums)
    {
      $where_sql .= "t.fid NOT IN ($unsearchforums) AND ";
    }
    
    // Group permission compatibility for >= MyBB 1.6
    if ($version['version_code'] >= 1600)
    {
    	$onlyusfids = array();
    
    	// Check group permissions if we can't view threads not started by us
    	$group_permissions = forum_permissions();
    	foreach($group_permissions as $fid => $forum_permissions)
    	{
    		if($forum_permissions['canonlyviewownthreads'] == 1)
    		{
    			$onlyusfids[] = $fid;
    		}
    	}
    	if(!empty($onlyusfids))
    	{
    		$where_sql .= " ((t.fid IN(".implode(',', $onlyusfids).") AND t.uid='{$mybb->user['uid']}') OR t.fid NOT IN(".implode(',', $onlyusfids).")) AND "; 
    	}
    }
    
    // Decide and make a where statement
    if (count($tids) > 0)
    {
      $where_sql .= 't.tid IN (' . implode(',', $tids) . ')';
    }
    else
    {
      $where_sql .= '1 < 0';
    }
    
    // Use mybb built-in search engine system
    $sid = md5(uniqid(microtime(), 1));
    $searcharray = array(
        "sid" => $db->escape_string($sid),
        "uid" => $mybb->user['uid'],
        "dateline" => TIME_NOW,
        "ipaddress" => $db->escape_string($session->ipaddress),
        "threads" => '',
        "posts" => '',
        "resulttype" => "threads",
        "querycache" => $db->escape_string($where_sql),
        "keywords" => ''
    );

    $plugins->run_hooks("search_do_search_process");
    $db->insert_query("searchlog", $searcharray);
    redirect("search.php?action=results&sid=".$sid, $lang->redirect_searchresults);
  } 
} 


function clean_exceptions($val)
{
  $val = (int) trim($val);
  
  return ($val > 0);
}
?>